(* :Title: Curvature *)

(* :Context: RMCPackages`Curvature` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to calculate the curvature,
unit tangent vector, and unit normal vector to any
parameterized curve in two-space or three-space. In 
addition, the user can plot any parameterized
curve in the plane, and show animations of its
oscilating circles and its evolute. 
*)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 5.0 *)

(* :History:
	V1.0 by Bruce F. Torrence, March, 2002.
	V1.1 October 2003 - updated for version 5.
	
*)

(* :Keywords: Curvature, UnitTangent, Unit Tangent, UnitNormal, Unit Normal,
Osculating Circle, Evolute *)

BeginPackage["RMCPackages`Curvature`",
	"Utilities`FilterOptions`"]

EvoluteMovie::usage=
"EvoluteMovie[r,{t,tmin,tmax}] produces an animation of the 
osculating circle of the curve r[t] as it moves about the curve
and traces the Evolute of the curve. The option Frames determines
the number of frames in the movie. Note that PlotRange should
generally be used to set a sensible plot range.";

\[Kappa]::usage=
"\[Kappa][r][t] gives the curvature of the parametric curve r with
paramter t.";

Norm::usage = 
"Norm[v] calculates the norm of the vector v.";

Frames::usage=
"Frames is an option for EvoluteMovie and OsculatingCircleMovie that determines the
number of frames in the animation. The default setting is 15.";

OsculatingCircleMovie::usage=
"OsculatingCircleMovie[r,{t,tmin,tmax}] produces an animation of the 
osculating circle of the curve r[t] as it moves about the curve. The 
option Frames determines
the number of frames in the movie. Note that PlotRange should
generally be used to set a sensible plot range.";

UnitNormal::usage=
"UnitNormal[r][t] gives the unit normal vector to the parameterized
curve r with parameter t. To use this command, first define the function r
with a command such as r[t_]:={t, 2t, t-1}.";

UnitTangent::usage=
"UnitTangent[r][t] gives the unit tangent vector to the parameterized
curve r with parameter t. To use this command, first define the function r
with a command such as r[t_]:={t, 2t, t-1}.";

OsculatingCircleMovie::badarg=
	"Yikes! Set `1`StyleFunction to a Graphics directive function (such as 'Hue') whose
	arguments will range from 0 to 1 as the `2`-values increase." ;  


Begin["`Private`"]

Options[OsculatingCircleMovie]={
	Frames->15,
	PlotRange->{{-4,4},{-4,4}}
	};

UnitTangent[r_][t_] :=Simplify[r'[t]/Norm[r'[t]],Element[t,Reals]];

UnitNormal[r_][t_]:=Simplify[UnitTangent[r]'[t]/Norm[UnitTangent[r]'[t]],Element[t,Reals]];

\[Kappa][r_][t_]:=Simplify[Norm[Cross[r'[t],r''[t]]]/(Norm[r'[t]]^3),Element[t,Reals]] /; Length[r[t]]==3;

\[Kappa][r_][t_]:=Simplify[Norm[Cross[Append[r'[t],0],Append[r''[t],0]]]/(Norm[r'[t]]^3),Element[t,Reals]] /; Length[r[t]]==2;
	
center[r_][t_]:=r[t]+\[Kappa][r][t]^(-1)*UnitNormal[r][t];	
	
radius[r_][t_]:=Graphics[{Line[{r[t],center[r][t]}]}];

osculatingCircle[r_][t_]:=Graphics[
	{Hue[.1],Circle[center[r][t],\[Kappa][r][t]^(-1)]}
	];
	
EvoluteMovie[r_,{t_,tmin_,tmax_},opts___Rule]:=Module[
{fr,pr,dk,pp,k,evopts,rad,cir,evo},
	{fr,pr}={Frames,PlotRange}/.{opts}/.Options[OsculatingCircleMovie];
	dk=(tmax-tmin)/(fr-1);
	pp=ParametricPlot[r[t]//Evaluate,{t,tmin,tmax},
		DisplayFunction->Identity,
		Evaluate[FilterOptions[ParametricPlot,opts]],
		AspectRatio->Automatic,
		PlotStyle->Hue[.7],
		PlotRange->pr,
		ImageSize->450];
	evopts={};
	Do[
		rad=radius[r][k];
		cir=osculatingCircle[r][k];
		evopts=Append[evopts,center[r][k]];
		evo=Graphics[{Hue[0],Line[evopts]}];
		Show[pp,rad,cir,evo,DisplayFunction->$DisplayFunction],
		{k,tmin,tmax,dk}];
	];

OsculatingCircleMovie[r_,{t_,tmin_,tmax_},opts___Rule]:=Module[
{fr,pr,dk,pp,k,rad,cir},
	{fr,pr}={Frames,PlotRange}/.{opts}/.Options[OsculatingCircleMovie];
	dk=(tmax-tmin)/(fr-1);
	pp=ParametricPlot[r[t]//Evaluate,{t,tmin,tmax},
		DisplayFunction->Identity,
		Evaluate[FilterOptions[ParametricPlot,opts]],
		AspectRatio->Automatic,
		PlotStyle->Hue[.7],
		PlotRange->pr,
		ImageSize->450];
	Do[
		rad=radius[r][k];
		cir=osculatingCircle[r][k];
		Show[pp,rad,cir,DisplayFunction->$DisplayFunction],
		{k,tmin,tmax,dk}];
	]


End[]

EndPackage[]