(* :Title: Eigenspace *)

(* :Context: RMCPackages`Eigenspace` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to make a plot of the span of 
a set of vectors in real 2-space or 3-space, and to make
a plot of the Eigenspace associated with each Eigenvalue of
a linear transformation of real 2-space or 3-space. 
*)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 4.0 *)

(* :History:
	V1.0 by Bruce F. Torrence, January 2001. 
*)

(* :Keywords: Eigenspace, Eigenvalue, Eigenvector, Eigentable, Span
	ShowSpan  *)


BeginPackage["RMCPackages`Eigenspace`", "Utilities`FilterOptions`"]

ShowSpan::usage = "ShowSpan[v1] or ShowSpan[v1,v2] will produce a plot of the 
subspace which is the span of v1 or of v1 and v2. Here v1 and v2 are vectors in 
2-space or 3-space (such as {1,2,1} or {0,2}). The options PointStyle, LineStyle, 
and PlaneStyle may be set to a Graphics or Graphics3D directive, or to a list of 
such directives (e.g. LineStyle->{Hue[.7], Thickness[.01]}). Any Graphics or 
Graphics3D option may also be used.";

Eigenspace::usage = "Eigenspace[matrix] will produce a plot of the 
eigenspace associated with each real eigenvalue of matrix, along with a 
table of eigenvalues and eigenvectors. The options PointStyle, 
LineStyle, and PlaneStyle may be set to a Graphics or Graphics3D 
directive, or to a list of such directives (e.g. LineStyle->{Hue[.7], Thickness[.01]}). 
Any Graphics or Graphics3D option may also be used.";

Eigentable::usage = "Eigentable[matrix] will produce a table showing the
real eigenvalues and eigenvectors for matrix.";

Options[Span2]={
	PointStyle->{Hue[1],PointSize[.02]},
	LineStyle->Hue[1], PlaneStyle->GrayLevel[0.65]
};	

Options[Span3]={
	PointStyle->{Hue[1], PointSize[.02]},
	LineStyle->Hue[1], PlaneStyle->GrayLevel[0.65]
};

Options[Eigen2Space]={
	Frame->True, FrameTicks->{{0},{0}},
	AspectRatio->1, PlotRange->{{-1,1},{-1,1}}
};

Options[Eigen3Space]={
	BoxRatios->{1,1,1}, PlotRange->{{-1,1},{-1,1},{-1,1}},
	Axes->True, Ticks->{{0},{0},{0}}, Lighting->False
};
	
Begin["`Private`"]  (* "RMCPackages`Eigenspace`Private`" *)

(* graphic object for span of a single vector in 2-space *)
Span2[v_List, opts___Rule] := Module[{ps,ls},
	{ps,ls}={PointStyle, LineStyle}/.{opts}/.Options[Span2];
	If[Head[ls]===List, ls=Sequence @@ ls];
	If[Head[ps]===List, ps=Sequence @@ ps];
	If[v == {0, 0}, 
		Graphics[{ps, Point[v]}, FilterOptions[Graphics,opts]//Evaluate], 
   		Graphics[{ls, Line[3*{-(v/Sqrt[v . v]), v/Sqrt[v . v]}]},  
    		FilterOptions[Graphics,opts]//Evaluate]
    	]
    ];

(* graphic object for span of a pair of vectors in 2-space *)
Span2[v1_List, v2_List, opts___Rule] := Module[{pts,ps},
	ps=PlaneStyle/.{opts}/.Options[Span2];
	If[Head[ps]===List, ps=Sequence @@ ps];
	pts = {{1, 1}, {-1, 1}, {-1, -1}, {1, -1}};
   	If[Length[NullSpace[{v1, v2}]] == 1, If[v1 == {0, 0}, Span2[v2, opts], Span2[v1, opts]], 
    	Graphics[{ps, Polygon[pts]}, FilterOptions[Graphics,opts]//Evaluate, PlotRange->{{-1,1},{-1,1}}]
    	]
  	];

(* graphic object for span of a single vector in 3-space *)
Span3[v_List, opts___Rule] := Module[{ps,ls},
	{ps,ls}={PointStyle, LineStyle}/.{opts}/.Options[Span3];
	If[Head[ls]===List, ls=Sequence @@ ls];
	If[Head[ps]===List, ps=Sequence @@ ps];
	If[v == {0, 0, 0}, 
		Graphics3D[{ps, Point[v]}, FilterOptions[Graphics3D,opts]//Evaluate], 
		Graphics3D[{ls, Line[3*{-(v/Sqrt[v . v]), v/Sqrt[v . v]}]},
			FilterOptions[Graphics3D,opts]//Evaluate]
		]
    ];
    
(* graphic object for span of a pair of vectors in 3-space *)
Span3[v1_List, v2_List, opts___Rule] := Module[
   	{ps,a, b, c, x, y, z, m, eq, j, k, pts},
   	ps=PlaneStyle/.{opts}/.Options[Span3];
	If[Head[ps]===List, ps=Sequence @@ ps]; 
   	If[Length[NullSpace[{v1, v2}]] == 2, 
   		If[v1 == {0, 0, 0}, Span3[v2, opts], Span3[v1, opts]], 
    	{a, b, c} = Cross[v1, v2]; 
    	m = Max[Abs[a], Abs[b], Abs[c]]; 
    	eq = a*x + b*y + c*z == 0; 
     	If[m == Abs[a], {j, k} = {y, z}, 
     		If[m == Abs[b], {j, k} = {x, z}, {j, k} = {x, y}]]; 
     	pts = {	{x, y, z} /. Solve[{j == 1, k == 1, eq}][[1]], 
       			{x, y, z} /. Solve[{j == -1, k == 1, eq}][[1]], 
       			{x, y, z} /. Solve[{j == -1, k == -1, eq}][[1]], 
       			{x, y, z} /. Solve[{j == 1, k == -1, eq}][[1]]}; 
     	Graphics3D[{ps, Polygon[pts]}, FilterOptions[Graphics3D,opts]//Evaluate]
     	]
     ];
     
ShowSpan[v_List, opts___Rule] := 
	If[Length[v] == 2, Show[Span2[v, opts], Evaluate[FilterOptions[Graphics,opts]], Options[Eigen2Space]], 
   		If[Length[v] == 3, Show[Span3[v, opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
   			Print["Incorrect Syntax - Input a VECTOR from 2-space or 3-space."]
   		]
   	]; 
   	
ShowSpan[v1_List, v2_List, opts___Rule] := 
	If[Length[v1] == 2, Show[Span2[v1, v2, opts], Evaluate[FilterOptions[Graphics,opts]], Options[Eigen2Space]], 
   		If[Length[v1] == 3, Show[Span3[v1, v2, opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
   			Print["Incorrect Syntax - Input 2 VECTORS from 2-space or 3-space."]
   		]
   	];     			       
     
mytable[xx_, yy_] := Module[{txt}, 
   	txt[x_] := StyleForm[x, FontFamily -> "Times", 
      	FontWeight -> "Bold", FontSize -> 12]; 
    TableForm[Transpose[{xx, yy}], TableDepth -> 2, 
     	TableHeadings -> {None, {txt["Eigenvalue"], txt["Eigenvector"]}}]
     ];  
     	
Eigentable[m_List] := Module[{zero, sys, i, li, type, p, re, rev}, 
   zero = Table[0, {i, Length[m[[1]]]}]; 
   sys = Eigensystem[m];
   li = sys[[1]]; 
   type = Head /@ N[li] /. Integer -> Real; 
   p = Position[type, Real]; 
   re = Extract[li, p]; (* real eigenvalues *)
   rev = Extract[sys[[2]], p]; (* real eigenvectors *)
   p = Position[rev, zero]; (* don't report zero vector as an eigenvector *)
   re = Delete[re, p]; 
   rev = Delete[rev, p]; 
   mytable[re, rev]
   ];
   
Eigen2Space[mat_List, opts___Rule] := Module[{sys, li, type, p, re, rev, n, m}, 
	sys = Eigensystem[mat];
	li = sys[[1]]; 
    type = Head /@ N[li] /. Integer -> Real; 
    p = Position[type, Real]; 
    re = Extract[li, p]; 
    rev = Extract[sys[[2]], p]; 
    p = Position[rev, {0, 0}]; 
    re = Delete[re, p]; 
    rev = Delete[rev, p]; 
    n = Length[re]; 
    m = Length[Union[re]]; 
    Which[	n == 1 || m == 2, 
    			Show[Span2[#,opts]& /@ rev, Evaluate[FilterOptions[Graphics,opts]], Options[Eigen2Space]], 
    		n == 2 && m == 1, 
     			Show[Span2[rev[[1]],rev[[2]],opts], Evaluate[FilterOptions[Graphics,opts]], Options[Eigen2Space]]]; 
     If[n >= 1, mytable[re, rev], Print["No real eigenvalues."]]]

Eigen3Space[mat_List, opts___Rule] := Module[{sys, li, type, p, re, rev, n, m, nullity}, 
  	sys = Eigensystem[mat];
	li = sys[[1]];  
  	type = Head /@ N[li] /. Integer -> Real; 
    If[Count[type, Real] > 0, 
    	p = Position[type, Real]; 
    	re = Extract[li, p]; 
    	rev = Extract[sys[[2]], p], (* begin else *)
    	sys = Eigensystem[N[mat]];
    	li = sys[[1]]; 
    	type = Head /@ N[li] /. Integer -> Real; 
    	p = Position[type, Real]; 
    	re = Extract[li, p]; 
    	rev = Extract[sys[[2]], p], (* end else *)
    	Null]; (* end if *)
    p = Position[rev, {0, 0, 0}]; 
    re = Delete[re, p]; 
    rev = Delete[rev, p]; 
    n = Length[re]; 
    m = Length[Union[re]]; 
    Which[
    	n == m, 
    		Show[Span3[#,opts]& /@ rev, Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
    	n == 2 && m == 1, 
     		Show[Span3[rev[[1]],rev[[2]],opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
     	n == 3 && m == 2, 
     		If[re[[1]] == re[[2]], 
      			Show[Span3[rev[[3]],opts], Span3[rev[[1]], rev[[2]],opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
      				If[re[[1]] == re[[3]], 
      					Show[Span3[rev[[2]],opts], Span3[rev[[1]], rev[[3]],opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]], 
       						Show[Span3[rev[[1]],opts], Span3[rev[[2]], rev[[3]],opts], Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]]]], 
       	n == 3 && m == 1, 
     		Show[Graphics3D[{Text["Eigenspace=\!\(\[DoubleStruckCapitalR]\^3\)", {0.3, 0, 1}]}, Evaluate[FilterOptions[Graphics3D,opts]], Options[Eigen3Space]]]]; 
    mytable[re, rev]
  ];   
  
Eigenspace[m_List, opts___Rule] := 
	Which[
		Length[m[[1]]] == 2, Eigen2Space[m, opts], 
		Length[m[[1]]] == 3, Eigen3Space[m, opts], 
   		Length[m[[1]]] >= 4, Eigentable[m]
   	]; 
   	
End[]

EndPackage[]