(* :Title: Parametric Plotting Tools *)

(* :Context: RMCPackages`ParametricPlottingTools` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to make directed parametric
plots (whose color changes as the parameter increases), 
animations of a ParametricPlot, and animations of a homotopy
between two real-valued functions of the unit interval. 
*)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 4.0 *)

(* :History:
	V1.0 by Bruce F. Torrence, January 2001. 
*)

(* :Keywords: vector-valued, vector valued, parametric, homotopy,
	DirectedParametricPlot, ParametricPlotMovie, HomotopyMovie  *)


BeginPackage["RMCPackages`ParametricPlottingTools`", 
	"Utilities`FilterOptions`", "Graphics`Arrow`"]

DirectedParametricPlot::usage = "DirectedParametricPlot[{x[t],y[t]}, {t,tmin,tmax}]
is essentially ParametricPlot, but it produces a color curve which is 
green at the beginning (green for go - when t is near tmin) and red at 
the end (red for stop - when t is near tmax). DirectedParametricPlot 
accepts the same options as ParametricPlot with the single exception 
that PlotStyle settings will have no effect. Specific options for
DirectedParametricPlot include ColorSegments (set to 8 by default) which 
controls the number of different shades along the curve from start to finish; 
and HueRange (set to {.3,1} by default) which controls the range of Hues 
along the curve.";

ParametricPlotMovie::usage="ParametricPlotMovie[{x[t],y[t]},{t,tmin,tmax}] 
uses the same syntax as the built-in command ParametricPlot, but produces
a movie instead of a static plot (double click any frame to animate). The movie 
illustrates how the curve is traced out in the plane as t moves linearly
from tmin to tmax. The option Frames controls the number of frames in
the movie. It is set to 20 by default, but may be set to any natural number.
The option setting LeftFrame->False will supress the left image on each
frame. The option setting ShowVector->True will replace the point on each
right image with a vector eminating from the origin. The option PointStyle
may be set to any graphics directive (or list thereof); it controls the
appearance of the moving point. ParametricPlotMovie will also accept any
options accepted by ParametricPlot (such as AspectRatio->Automatic).
ParametricPlotMovie[{x[t],y[t],z[t]},{t,tmin,tmax}] will produce a similar
graphic, but with a 3D parametric plot in the right pane. In this case, one
may use any option accepted by ParametricPlot3D.";
	
HomotopyMovie::usage = "HomotopyMovie[{x[s,t],y[s,t]},{s,t}] generates a movie
illustrating the homotopy {x[s,t],y[s,t]} mapping the unit square [0,1]^2
into the plane. i.e. the homotopy is between {x[0,t],y[0,t]} and {x[1,t],y[1,t]}.
The option Frames controls the number of frames in the movie. It is set to 15 
by default, but may be set to any natural number. HomotopyMovie will also 
accept any options accepted by ParametricPlot; it is a good idea to use
the option setting PlotRange->{{xmin,xmax},{ymin,ymax}} so that the image
on the right will cover the same portion of the plane in each frame. You 
may use the syntax HomotopyMovie[{x[s,t],y[s,t],{s},{t,tmin,tmax}] to make
a movie of a homotopy between functions defined on the interval [tmin,tmax].";

Options[DirectedParametricPlot] = 
	{ColorSegments->8, HueRange->{0.3, 1.}, PlotPoints->25}; 

Options[LeftFrame]={PointStyle->{PointSize[.04],Hue[1]},
      PlotStyle->Hue[.7]};

Options[RightFrame2D]={PointStyle->{PointSize[0.04],Hue[1]},
      ShowVector->False};

Options[RightFrame3D]={PointStyle->{PointSize[0.04],Hue[1]},
      ShowVector->False,PlotStyle->Hue[.7]};

Options[ParametricPlotMovie] = {Frames->20, LeftFrame->True, ImageSize->600};

Options[SqGraphic] = {Segments->15, PlotStyle->{Hue[1],Thickness[.01]}}; 

Options[HomotopyFrame] = {Frames->15, PlotStyle->{Hue[1],Thickness[.01]}}; 

Options[HomotopyMovie] = {Frames->15, 
	PlotStyle->{Hue[1],Thickness[.01]},
	ImageSize->600}; 


Begin["`Private`"]  (* "RMCPackages`ParametricPlottingTools`Private`" *)


DirectedParametricPlot[f_, {t_, tmin_, tmax_}, opts___Rule] := 
  Module[{colorsegs, huerange, plotpts, pplot, data, m, directives}, 
   	{colorsegs, huerange, plotpts} = 
     {ColorSegments, HueRange, PlotPoints} /. {opts} /. 
      Options[DirectedParametricPlot]; 
	plotpts = Max[colorsegs, plotpts]; 
    pplot = ParametricPlot[f, {t, tmin, tmax}, DisplayFunction -> 
       	Identity, PlotStyle -> Automatic, PlotPoints -> plotpts, 
      	Evaluate[FilterOptions[ParametricPlot, opts]]]; 
    data = Nest[First, pplot, 4]; 
	m = Round[Length[data]/colorsegs]; 
    data = Join[Partition[data, m + 1, m], 
      	{Take[data, -Min[Length[data], m + 1]]}]; 
	m = Length[data] - 1; 
    If[m == 0, directives = {Hue[huerange[[1]]]}, 
     	directives = Table[Hue[huerange[[1]]*(m - t)/m + huerange[[2]]*t/m], 
       	{t, 0, m}]]; 
	pplot = pplot /. {{Line[_]}} -> 
		MapThread[{#1, Line[#2]} & , {directives, data}]; 
    Show[pplot, DisplayFunction -> $DisplayFunction]
  ]


(*LeftFrame is the graphic forming the left hand side of the graphics array
in a single frame of a ParametricPlotMovie*)

  LeftFrame[tt_,{tmin_,tmax_},opts___Rule]:=
  Module[{ps=PointStyle/.{opts}/.Options[LeftFrame],
      plotsty=PlotStyle/.{opts}/.Options[LeftFrame],pr=tmax-tmin},
    If[Head[ps]===List,ps=Sequence@@ps];
    If[Head[plotsty]===List,plotsty=Sequence@@plotsty];
    Graphics[{{plotsty,Line[{{tmin,0},{tmax,0}}]},{ps,Point[{tt,0}]},
        Text[tmin,{tmin,-pr/5}],Text[tmax,{tmax,-pr/5}]},
      PlotRange->{-pr,pr}]
      ]
	


(*RightFrame2D is the graphic forming the right hand side of the graphics
array in a single frame of a 2D ParametricPlotMovie*)

  RightFrame2D[tt_,fcn_,{t_,tmin_,tmax_},opts___Rule]:=
  Module[{ps,
      sv},{ps,sv}={PointStyle,ShowVector}/.{opts}/.Options[RightFrame2D];
    If[Head[ps]===List,ps=Sequence@@ps];
    If[sv==False,
      ParametricPlot[fcn,{t,tmin,tmax},DisplayFunction->Identity,
        Epilog->{ps,Point[fcn/.t->tt]},
        Evaluate[FilterOptions[ParametricPlot,opts]],
        PlotStyle->Hue[0.7]],
      ParametricPlot[fcn,{t,tmin,tmax},DisplayFunction->Identity,
        Epilog->{ps,Arrow[{0,0},fcn/.t->tt]},
        Evaluate[FilterOptions[ParametricPlot,opts]],
        PlotStyle->Hue[0.7]]
      ]
    ]
	

(*RightFrame3D is the graphic forming the right hand side of the graphics \
array in a single frame of a 3D ParametricPlotMovie*)

  RightFrame3D[tt_,{xt_,yt_,zt_},{t_,tmin_,tmax_},opts___Rule]:=
  Module[{ps,sv,plotsty,g1},{ps,sv,plotsty}=
      {PointStyle,ShowVector,PlotStyle}/.{opts}/.Options[RightFrame3D];
    If[Head[ps]===List,ps=Sequence@@ps];
    If[sv==False,
      g1=FullForm[
            ParametricPlot3D[{xt,yt,zt,plotsty},{t,tmin,tmax},
              DisplayFunction->Identity,
              Evaluate[FilterOptions[ParametricPlot3D,opts]]]][[1,1]];
      g1=Append[g1,{ps,Point[{xt,yt,zt}/.t->tt]}];
      Graphics3D[g1,DisplayFunction->Identity,
        Evaluate[FilterOptions[Graphics3D,opts]]],
      g1=FullForm[
            ParametricPlot3D[{xt,yt,zt,plotsty},{t,tmin,tmax},
              DisplayFunction->Identity,
              Evaluate[FilterOptions[ParametricPlot3D,opts]]]][[1,1]];
      g1=Append[
          g1,{ps,Point[{xt,yt,zt}/.t->tt],
            Line[{{0,0,0},{xt,yt,zt}/.t->tt}]}];
      Graphics3D[g1,DisplayFunction->Identity,
        Evaluate[FilterOptions[Graphics3D,opts]]]
      ]
    ]




ParametricPlotMovie[{xt_,yt_},{t_,tmin_,tmax_},opts___Rule]:=
  Module[{fr,lf, ,is,dt},
  	{fr,lf,is}=
        {Frames,LeftFrame,ImageSize}/.{opts}/.Options[ParametricPlotMovie];
    dt=(tmax-tmin)/fr;
    If[lf==True,
      Do[Show[GraphicsArray[{LeftFrame[k,{tmin,tmax},opts],
              RightFrame2D[k,{xt,yt},{t,tmin,tmax},opts]},
            GraphicsSpacing->0.2,ImageSize->is]],{k,tmin,tmax,dt}],
      Do[Show[RightFrame2D[k,{xt,yt},{t,tmin,tmax},opts],
          DisplayFunction->$DisplayFunction,ImageSize->is],{k,tmin,tmax,dt}]
      ]
    ]
    


ParametricPlotMovie[{xt_,yt_,zt_},{t_,tmin_,tmax_},opts___Rule]:=
  Module[{fr,lf,is,dt},
  {fr,lf,is}=
  	{Frames,LeftFrame,ImageSize}/.{opts}/.Options[ParametricPlotMovie];
    dt=(tmax-tmin)/fr;
    If[lf==True,
      Do[Show[GraphicsArray[{LeftFrame[k,{tmin,tmax},opts],
              RightFrame3D[k,{xt,yt,zt},{t,tmin,tmax},opts]},
            GraphicsSpacing->0.2,ImageSize->is]],{k,tmin,tmax,dt}],
      Do[
        Show[RightFrame3D[k,{xt,yt,zt},{t,tmin,tmax},opts],
          DisplayFunction->$DisplayFunction,ImageSize->is],{k,tmin,tmax,dt}]
      ]
    ]
    
    
    	
(* SqGraphic is the graphic in the left side of each frame of HomotopyMovie *)

SqGraphic[t_, opts___Rule] := 
  Module[{segs,ps,hlines,colorline},
	{segs,ps} = {Segments,PlotStyle} /. {opts} /. Options[SqGraphic];
	If[Head[ps] === List, ps = Sequence @@ ps]; 
   	hlines = {GrayLevel[0.6], Table[Line[{{0, k}, {1, k}}], {k, 0, 1, 1/segs}]}; 
	If[t <= segs, 
     colorline = Line[{{0, k}, {1, k}}] /. k -> t/segs; 
     Graphics[{hlines, {ps, colorline}}, 
       AspectRatio -> Automatic], Graphics[hlines, AspectRatio -> Automatic]
	]
  ]
  

(* HomotopyFrame is the graphic in the right side of each frame of HomotopyMovie *)

HomotopyFrame[h_, fcn_, {s_}, {t_, tmin_, tmax_}, opts___Rule] := 
  Module[{fr,ps},
	{fr,ps} = {Frames,PlotStyle} /. {opts} /. Options[HomotopyFrame]; 
   	ParametricPlot[Evaluate[fcn /. s -> h/fr], {t, tmin, tmax}, 
    	DisplayFunction -> Identity, PlotStyle->ps,
		Evaluate[FilterOptions[ParametricPlot, opts]], 
		Ticks -> {None, None}]
   ]	

HomotopyMovie[fcn_, {s_}, {t_, tmin_, tmax_}, opts___Rule] := 
  Module[{fr,ps,is},
	{fr,ps,is} = {Frames,PlotStyle,ImageSize} /. {opts} /. Options[HomotopyMovie]; 
   	Do[Show[GraphicsArray[{
		SqGraphic[h, Segments -> fr, PlotStyle->ps], 
       	HomotopyFrame[h, fcn, {s}, {t, tmin, tmax}, opts]}, 
      	GraphicsSpacing -> 0.6], ImageSize->is
	   ], {h, 0, fr}]
  ]
  
HomotopyMovie[fcn_, {s_}, {t_}, opts___Rule] :=  
	HomotopyMovie[fcn, {s}, {t, 0, 1}, opts]
	
HomotopyMovie[fcn_, {s_,t_}, opts___Rule] :=
	HomotopyMovie[fcn, {s}, {t, 0, 1}, opts] 

End[]

EndPackage[]