(* :Title: Riemann Sum *)

(* :Context: RMCPackages`RiemannSum` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package defines the command RiemannSum which calculates a
Riemann sum, and produces a graphic illustrating the rectangles
whose area is the sum.
*)

(* :Package Version: 1.21 *)

(* :Mathematica Version: 4.1 *)

(* :History:
	1.0 by Bruce F. Torrence, October, 2001.
	1.1 same guy, January, 2002. 
	1.2 same guy, April, 2002.
	1.21, me again, June, 2002....Printout->Long fixed; add fill to rectangles.
*)

(* :Keywords: Riemann sum, RiemannSum, numerical integration.  *)

(* :Known Bugs: Option setting Printout->Long does not work.  *)


BeginPackage["RMCPackages`RiemannSum`", "Utilities`FilterOptions`"]

RiemannSum::usage = "RiemannSum[f,{x,xmin,xmax,dx}]
produces a graph of the function f over the interval
xminxxmax and includes rectangles for the
Riemann sum approximation of the definite integral of f, using
widths dx. Alternatively, one may use the syntax 
RiemannSum[f,{x,xmin,xmax},Subintervals->n] to specify that
there should be n subintervals of equal width. The option
Subintervals may also be set to a list of values for the endpoints
of the subintervals. This allows
for rectangles of unequal width. Other options include Height,
which may be set to Left, Right, Midpoint (default), Upper, or Lower.
RiemannSum will also return the numerical value of the approximation
unless the option Printout is set to False. In 
the case of Upper or Lower sums, each subinterval is sampled many
times to determine where the min or max occurs; the number of points
sampled may be adjusted with the option ULSubdivisions. Lastly, the
option RectangleStyle may be set to a Graphics directive (or list
of directives) to control the appearance of the rectangles.";

Height::usage = "Height is an option for RiemannSum. It may be set
to Midpoint (default), Left, Right, Upper, or Lower."

Subintervals::usage = "Subinterals is an option for RiemannSum. Set it 
to a positive integer n to get n rectangles. Set it to a list of interval
endpoints to get rectangles with these endpoints. For instance,
Subintervals->Table[Random[Real,{xmin,xmax}],{15}] gives randomly
chosen subinterval endpoints."

RectangleStyle::usage = "RectangleStyle is an option for RiemannSum. Set
it to a Graphics directive (or to a list of Graphics directives) to be
applied to the lines comprising the edges of the rectangles in the graphic."

FillStyle::usage = "FillStyle is an option for RiemannSum. Set
it to a Graphics directive (or to a list of Graphics directives) to determine
the style of the interiors of the rectangles."

ULSubdivisions::usage = "ULSubdivisions is an option for RiemannSum that
is only relevant if the option Height is set to Upper or Lower. In these
cases, ULSubdivisions controls how many points are sampled in each
subinterval to determine the function's max or min value. The default
setting is Automatic, which sets the numerical 
value according to how many rectangles are present in the plot (fewer
rectangles lead to more points sampled per subinterval)."

Printout::usage = "Printout is an option for RiemannSum that controls what
will be printed along with the graphic. Set it to False to supress any
textual output. By default it is set to True, which prints only the
value of the Riemann sum. Set it to Long for a full report."

Long::usage = "Long is a setting for the option Printout in the command
RiemannSum."

Upper::usage = "Upper is a setting for the option Height in the command
RiemannSum."

Lower::usage = "Lower is a setting for the option Height in the command
RiemannSum."

Options[RiemannSum]={
			Height->Midpoint, 
			Subintervals->15,
			PlotStyle->{RGBColor[0,0,1],Thickness[.004]},
			RectangleStyle->GrayLevel[0],
			FillStyle->GrayLevel[.8],
			ULSubdivisions->Automatic,
			Printout->True
			};
			
			
RiemannSum::badarg1="No go, Joe. You need numeric bounds on `1`.";

RiemannSum::badarg2="Something is wrong with your input. Check it carefully
	and try again. You can type ?RiemannSum to get information on the correct
	use of this command.";
	
RiemannSum::badHeight="Something is wrong with Height specification. You can
	type ?RiemannSum to get information on the correct use of this option."

Begin["`Private`"]  (* "RMCPackages`RiemannSum`Private`" *)

subdivide[{a_,b_}, n_]:=Module[{dx=(N[b]-a)/n, i},
	Table[a+i*dx,{i,0,n}]
	];

RiemannSum[f_,{x_, xmin_?NumericQ, xmax_?NumericQ, delx_:Automatic}, opts___Rule] := 
  Module[{dx=delx, subint,ht,xvals,i,uls,fullxlist,ypts,ps,rs,fs,fplot,biglist,g,
  ptlst,recs,recfill,pntout}, 
  	{subint,ht}={Subintervals,Height}/. {opts} /. Options[RiemannSum];
	
	(* xvals are the subinterval endpoints on the x axis *)
	xvals=Which[
			NumericQ[dx]&&subint==Subintervals/.Options[RiemannSum],
				Union[N[Table[i,{i,xmin,xmax,dx}]],{N[xmax]}],
			NumericQ[dx]&&IntegerQ[subint]&&subint>=1,
				Union[N[Table[xmin+i*dx,{i,0,subint-1}]],{N[xmax]}],
			dx==Automatic&&IntegerQ[subint]&&subint>=1,
				subdivide[{xmin,xmax},subint],	
			Head[subint]===List,
				Union[{N[xmin]},N[subint],{N[xmax]}],
			True, 
				Message[RiemannSum::badarg2]; Return[]
			];	
   	
   	(* in event of upper or lower sums, how many points are sampled
   	   in each subinterval to determine max/min value? *)
   	subint=Length[xvals]-1;
   	uls = ULSubdivisions /. {opts} /. Options[RiemannSum];
   	If[uls===Automatic,
   		  Which[subint>=20,uls=2,
   				subint>=10,uls=5,
   				subint>=5,uls=9,
   				True,uls=20
   				]
   	];
   	
   	(* ypts are the values of f that are the heights of the rectangles;
   	   length of ypts is one less than length of xvals *)

	ypts=Which[
			ht===Midpoint,
				f/.x->0.5(Drop[xvals,-1]+Drop[xvals,1]),
			ht===Left,
				f/.x->Drop[xvals,-1],
			ht===Right,
				f/.x->Drop[xvals,1],	
			ht===Upper,
				fullxlist=(subdivide[#,uls]&/@Partition[xvals,2,1]);
				Flatten[Max/@(f/.x->fullxlist)],
			ht===Lower,
				fullxlist=(subdivide[#,uls]&/@Partition[xvals,2,1]);
				Flatten[Min/@(f/.x->fullxlist)],
			True,
				Message[RiemannSum::badHeight]; Return[]
			];	
    
    (* Decide what to print out with the graphic. *)
    pntout = Printout /. {opts} /. Options[RiemannSum];
 	
 	Which[
 			pntout===True,
 				Print[ht," Riemann Sum = ", 
        			Apply[Plus,(Drop[xvals,1]-Drop[xvals,-1])*ypts]],
        	pntout===Long,
        		Print["subinterval endpoints = ", xvals];
 				Print["ULSubdivisions = ", uls];
 				Print["Height = ", ht];
				Print["rectangle heights = ", ypts];
				Print[ht," Riemann Sum = ", 
          			Apply[Plus,(Drop[xvals,1]-Drop[xvals,-1])*ypts]]
 		];	
 	
	{ps,rs,fs} = {PlotStyle,RectangleStyle,FillStyle} /. {opts} /. Options[RiemannSum]; 
    rs = If[Head[rs] =!= List, {rs}, rs];
    fs = If[Head[fs] =!= List, {fs}, fs]; 
    fplot = Plot[f, {x, xmin, xmax}, 
     	DisplayFunction -> Identity, PlotStyle -> ps, 
		Evaluate[FilterOptions[Plot, opts]]];
	biglist=Transpose[{Drop[xvals,-1],ypts,Drop[xvals,1]}];	 
	g[a_,b_,c_]:={{a,0},{a,b},{c,b}};
	ptlst=Apply[g,#]&/@biglist;
	ptlst=Append[Flatten[ptlst,1],{Last[xvals],0}];
	recs=Graphics[Append[rs,Line[ptlst]]];
	g[a_,b_,c_]:={{a,0},{c,b}};
	ptlst=Apply[g,#]&/@biglist;
	recfill=Append[fs,Apply[Rectangle,#]&/@ptlst];
	Show[fplot,recs,
		DisplayFunction->$DisplayFunction, 
		Prolog->{recfill},
		Epilog->{GrayLevel[0],Line[{{xmin,0},{xmax,0}}]}
		]
]

RiemannSum[f_,{x_, xmin_, xmax_, delx_:Automatic}, opts___Rule] :=	
	Message[RiemannSum::badarg1, x];


End[]

EndPackage[]