(* :Title: Slice Surface *)

(* :Context: RMCPackages`SliceSurface` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to plot any level surface
f[x,y,z]==c by rendering the surface's intersections
with planes orthogonal to the x, y and z axes. 
*)

(* :Package Version: 2.3 *)

(* :Mathematica Version: 4.2 *)

(* :History:
	V1.0 by Bruce F. Torrence, August 1999.
	V1.1 by Bruce F. Torrence, August 2000. 
	V2.0 by the same guy, October 2001.
	V2.1 and 2.2 by BFT, February 2002.
	V2.3 by BFT, April 2003.
*)

(* :Keywords: 3D, plot, surface, level surface, contour lines *)


BeginPackage["RMCPackages`SliceSurface`",
	"Graphics`ImplicitPlot`",
	"Utilities`FilterOptions`"]
	
Unprotect[
	SliceSurface,
	XContours,
	YContours,
	ZContours,
	XContourStyle,
	YContourStyle,
	ZContourStyle,
	XStyleFunction,
	YStyleFunction,
	ZStyleFunction
	]	


SliceSurface::usage = 
"SliceSurface[f==g,{x,xmin,xmax},{y,ymin,ymax},{z,zmin,zmax}] produces a 3D \
plot of the surface described by the equation in 3 variables f==g. For \
instance, the equation might be x^2+y^2-z^2==4. The plot consists of \
level-curves in the x, y, and z directions; i.e. it consists of contours \
produced by the intersection of the surface with planes z=z1, z=z2, ... , \
y=y1, y=y2, ... , x=x1, x=x2, ... SliceSurface may also be used as a
utility for graphing a real-valued function g[x,y] of two variables - in this
case the syntax may be simplified to match that of Plot3D. That is, one may use
SliceSurface[g,{x,xmin,xmax},{y,ymin,ymax}]. There are several 
important options in either case.
The option XContours controls the number of \
contours in the x-direction. It may be set to a number or a list {x1,x2,...}. \
YContours and ZContours behave similarly. The option XContourStyle controls \
the color, thickness, etc. of the x-contours. It may be set to a Graphics3D \
directive or a list of such directives, and affects all contour lines in the
x-direction. YContourStyle and ZContourStyle \
behave similarly. One may also apply different styles to the contours in
the x-direction (for example) by using the option XStyleFunction. Set this
option to a Graphic directive function, and the values of this directive function with
agruments increasing from 0 and 1 will be applied to the various x-contours. Try
'XStyleFunction->Hue, YContours->0, ZContours->0' to get a feel for it. 
SliceSurface can also take any options handled by \
ImplicitPlot (in the Graphics`ImplicitPlot` package). For instance, the \
option PlotPoints->40 might be applied to smooth out any jaggies in the contours. \
Finally, SliceSurface will also intelligently handle any Graphics3D options \
you may wish to apply to the final plot.";

XContours::usage = 
"XContours is an option for SliceSurface. XContours->n produces a plot with n \
contour curves normal to the x-direction. It may be set to 0 if you desire a \
plot with no such contours. XContours->{x1,x2,...}produces contours at the \
values x=x1, x=x2,...";

YContours::usage = 
"YContours is an option for SliceSurface. YContours->n produces a plot with n \
contour curves normal to the y-direction. It may be set to 0 if you desire a \
plot with no such contours. YContours->{y1,y2,...}produces contours at the \
values y=y1, y=y2,...";

ZContours::usage = 
"ZContours is an option for SliceSurface. ZContours->n produces a plot with n \
contour curves normal to the z-direction. It may be set to 0 if you desire a \
plot with no such contours. ZContours->{z1,z2,...}produces contours at the \
values z=z1, z=z2,...";

XContourStyle::usage = 
"XContourStyle is an option for SliceSurface. XContourStyle may be set to a \
Graphics3D directive or a list of such directives. The directive(s) will be \
applied to the x-contours of the resulting plot. E.g. \
XConoturStyle->RGBBColor[0,1,0] will produce green contours in the \
x-direction."; 

YContourStyle::usage = 
"YContourStyle is an option for SliceSurface. YContourStyle may be set to a \
Graphics3D directive or a list of such directives. The directive(s) will be \
applied to the y-contours of the resulting plot. E.g. \
YConoturStyle->RGBBColor[0,1,0] will produce green contours in the \
y-direction." ;

ZContourStyle::usage = 
"ZContourStyle is an option for SliceSurface. ZContourStyle may be set to a \
Graphics3D directive or a list of such directives. The directive(s) will be \
applied to the z-contours of the resulting plot. E.g. \
ZConoturStyle->RGBBColor[0,1,0] will produce green contours in the \
z-direction." ;

XStyleFunction::usage = 
"XStyleFunction is an option for SliceSurface which can be used to apply various
settings of a Graphics directive to the different XContours. The values of the
function range from 0 to 1 as the X values in the plot increase. For example, the
setting 'XStyleFunction->Hue' will produce XContours whose Hue ranges from 0 to 1
as the countour values increase. The setting 'XStyleFunction->{Hue,Thickness[.02#]&}'
will in addition change the thickness of the XContours from 0 to .02."

YStyleFunction::usage = 
"YStyleFunction is an option for SliceSurface which can be used to apply various
settings of a Graphics directive to the different YContours. The values of the
function range from 0 to 1 as the Y values in the plot increase. For example, the
setting 'YStyleFunction->Hue' will produce YContours whose Hue ranges from 0 to 1
as the countour values increase. The setting 'YStyleFunction->{Hue,Thickness[.02#]&}'
will in addition change the thickness of the YContours from 0 to .02."

ZStyleFunction::usage = 
"ZStyleFunction is an option for SliceSurface which can be used to apply various
settings of a Graphics directive to the different ZContours. The values of the
function range from 0 to 1 as the Z values in the plot increase. For example, the
setting 'ZStyleFunction->Hue' will produce ZContours whose Hue ranges from 0 to 1
as the countour values increase. The setting 'ZStyleFunction->{Hue,Thickness[.02#]&}'
will in addition change the thickness of the ZContours from 0 to .02."


SliceSurface::badarg1=
    "Oops! You have specified a `1`StyleFunction option, and have fewer than
    two `1`Contours. Add more contours so that style function can be applied.";
    
SliceSurface::badarg2=
	"Oh-oh! Set `1`Contours to a pos. integer or to a list of `2`-values." ;  

SliceSurface::badarg3=
	"Yikes! Set `1`StyleFunction to a Graphics directive function (such as 'Hue') whose
	argument will range from 0 to 1 as the `2`-values increase." ;  


Begin["`Private`"]

Options[SliceSurface]={
		XContourStyle->RGBColor[.1,.3,.5],
		YContourStyle->RGBColor[.9,.1,.1],
		ZContourStyle->GrayLevel[.5],
		XContours->9, 
		YContours->9,
		ZContours->25,
		XStyleFunction->{},
		YStyleFunction->{},
		ZStyleFunction->{},
		BoxRatios->{1,1,1}};
		
SliceSurfaceG3D[f_==g_,
	{x_,xmin_,xmax_},
	{y_,ymin_,ymax_},
	{z_,zmin_,zmax_},
	opts___Rule]:=
	Module[{xcs,ycs,zcs,xc,yc,zc,br,delx,dely,delz,xvals,yvals,zvals,cgx,cgy,cgz,
	xlines,ylines,zlines,nx,ny,nz,xsf,ysf,zsf,xdir,ydir,zdir,j},
		{xcs,ycs,zcs,xc,yc,zc,xsf,ysf,zsf,br}=
		{XContourStyle,YContourStyle,ZContourStyle,XContours, 
			YContours,ZContours,XStyleFunction,YStyleFunction,
			ZStyleFunction,BoxRatios}/. {opts} /. Options[SliceSurface];
		
		xcs=Switch[xcs,_List,xcs,_,List[xcs]];
		ycs=Switch[ycs,_List,ycs,_,List[ycs]];
		zcs=Switch[zcs,_List,zcs,_,List[zcs]];
		
		xsf=Switch[xsf,_List,xsf,_Symbol,List[xsf],_Function,List[xsf],_,Message[SliceSurface::badarg3,"X","x"];Return[]];
		ysf=Switch[ysf,_List,ysf,_Symbol,List[ysf],_Function,List[ysf],_,Message[SliceSurface::badarg3,"Y","y"];Return[]];
		zsf=Switch[zsf,_List,zsf,_Symbol,List[zsf],_Function,List[zsf],_,Message[SliceSurface::badarg3,"Z","z"];Return[]];	
		
		xvals=Which[Head[xc]===List,xc,
					IntegerQ[xc] && xc>=2,
						delx=(xmax-xmin)/(xc-1);
						Table[xmin+i*delx,{i,0,xc-1}],
					xc==0,{},
					xc==1,{.5(xmax-xmin)},
					True, Message[SliceSurface::badarg2,"X","x"];Return[]; 
				   ];
		yvals=Which[Head[yc]===List,yc,
					IntegerQ[yc] && yc>=2,
						dely=(ymax-ymin)/(yc-1);
						Table[ymin+i*dely,{i,0,yc-1}],
					yc==0,{},
					yc==1,{.5(ymax-ymin)},
					True, Message[SliceSurface::badarg2,"Y","y"];Return[]; 
				   ];
		zvals=Which[Head[zc]===List,zc,
					IntegerQ[zc] && zc>=2,
						delz=(zmax-zmin)/(zc-1);
						Table[zmin+i*delz,{i,0,zc-1}],
					zc==0,{},
					zc==1,{.5(zmax-zmin)},
					True, Message[SliceSurface::badarg2,"Z","z"];Return[]; 
				   ];
		
				   
		nx = Length[xvals];
		ny = Length[yvals];
		nz = Length[zvals];	
		
		If[nx<2 && Length[xsf]>0,Message[SliceSurface::badarg1,"X"];Return[]];
		If[ny<2 && Length[ysf]>0,Message[SliceSurface::badarg1,"Y"];Return[]];
		If[nz<2 && Length[zsf]>0,Message[SliceSurface::badarg1,"Z"];Return[]];
		
		cgx=Map[
			ImplicitPlot[f==g/.x->#,{y,ymin,ymax},{z,zmin,zmax},
            	FilterOptions[ImplicitPlot,opts],DisplayFunction->Identity]&,
     		xvals];

		cgy=Map[
			ImplicitPlot[f==g/.y->#,{x,xmin,xmax},{z,zmin,zmax},
            	FilterOptions[ImplicitPlot,opts],DisplayFunction->Identity]&,
     		yvals];

		cgz=Map[
			ImplicitPlot[f==g/.z->#,{x,xmin,xmax},{y,ymin,ymax},
            	FilterOptions[ImplicitPlot,opts],DisplayFunction->Identity]&,
     		zvals];
		
		xlines=Map[Graphics[#][[1]]&,cgx];
		ylines=Map[Graphics[#][[1]]&,cgy];
		zlines=Map[Graphics[#][[1]]&,cgz];
				
		xlines3D = Table[xlines[[j]] /. 
			{___,Line[pts_]} :> 
				{Sequence@@Join[xcs,Map[#[(j-1.)/(nx-1)]&, xsf]],Line[Map[Insert[#,xvals[[j]],1]&,pts]]},
			{j,nx}];

		ylines3D = Table[ylines[[j]] /. 
			{___,Line[pts_]} :> 
				{Sequence@@Join[ycs,Map[#[(j-1.)/(ny-1)]&, ysf]],Line[Map[Insert[#,yvals[[j]],2]&,pts]]},
			{j,ny}];

		zlines3D = Table[zlines[[j]] /. 
			{___,Line[pts_]} :> 
				{Sequence@@Join[zcs,Map[#[(j-1.)/(nz-1)]&, zsf]],Line[Map[Insert[#,zvals[[j]],3]&,pts]]},
			{j,nz}];
		
		Graphics3D[{xlines3D,ylines3D,zlines3D},
			FilterOptions[Graphics3D,opts],Axes->True,BoxRatios->br,
      		DisplayFunction->$DisplayFunction]
      		];
				
	SliceSurface[f_==g_,
		{x_,xmin_,xmax_},
		{y_,ymin_,ymax_},
		{z_,zmin_,zmax_},opts___Rule]:=
		Show[
			SliceSurfaceG3D[f==g,
				{x,xmin,xmax},
				{y,ymin,ymax},
				{z,zmin,zmax},opts]
			];
			
	SliceSurface[f_,
		{x_,xmin_,xmax_},
		{y_,ymin_,ymax_},
		{z_,zmin_,zmax_},opts___Rule]:=
		Show[
			SliceSurfaceG3D[f==z,
				{x,xmin,xmax},
				{y,ymin,ymax},
				{z,zmin,zmax},opts]
			];
			
	SliceSurface[f_,
		{x_,xmin_,xmax_},
		{y_,ymin_,ymax_},
		opts___Rule]:=Module[{dx,dy,t,x0,y0,zmin,zmax,dz,z},
			dx=(xmax-xmin)/6;
			dy=(ymax-ymin)/6;
			t=Table[f/.{x->x0,y->y0},
				{x0,xmin,xmax,dx},{y0,ymin,ymax,dy}]//Flatten;
			zmin=Min[t];
			zmax=Max[t];
			dz=Max[zmax-zmin,.5];
			zmin=zmin-.2dz;
			zmax=zmax+.2dz;
			Show[
				SliceSurfaceG3D[f==z,
					{x,xmin,xmax},
					{y,ymin,ymax},
					{z,zmin,zmax},opts]
				]	
			]


End[]

Protect[
	SliceSurface,
	XContours,
	YContours,
	ZContours,
	XContourStyle,
	YContourStyle,
	ZContourStyle,
	XStyleFunction,
	YStyleFunction,
	ZStyleFunction
	]	

EndPackage[]